@@ -13,6 +13,8 @@ from .specs import (
     Indices,
     MediaTypes,
     OCIMediaTypes,
+    QuayAuthentication,
+    RedHatAuthentication,
 )
 
-__version__ = "0.1.15"
+__version__ = "0.2.0"
